/*
 * This Source Code is subject to the terms of the Mozilla Public License
 * version 2.0 (the "License"). You can obtain a copy of the License at
 * http://mozilla.org/MPL/2.0/.
 */

@namespace url("http://www.mozilla.org/keymaster/gatekeeper/there.is.only.xul");

scrollbox
{
  overflow: auto;
}

#groupDisabledWarning
{
  color: red;
}

dialog:not([advancedMode=true]) #nodes-tree,
dialog:not([advancedMode=true]) #nodes-tree-splitter
{
  display: none;
}

treechildren::-moz-tree-cell-text(anchor)
{
  font-weight: bold;
}
treechildren::-moz-tree-row(anchor, selected-false)
{
  background-color: #FFC0C0;
}

.attribute
{
  padding: 5px;
}
.attribute > checkbox:focus + vbox
{
  outline: 1px dotted ThreeDDarkShadow;
}
.attribute .label
{
  -moz-user-focus: ignore;
}
.attribute.advanced .checkbox-label-box
{
  display: none;
}
