pref("extensions.elemhidehelper.selectelement_key", "Accel Shift K, Accel Shift S, Accel Shift F3");
pref("extensions.elemhidehelper.showhelp", true);
pref("extensions.elemhidehelper.acceptlocalfiles", false);
pref("extensions.elemhidehelper.composer_defaultDomain", 3);
