/*
 * This Source Code is subject to the terms of the Mozilla Public License
 * version 2.0 (the "License"). You can obtain a copy of the License at
 * http://mozilla.org/MPL/2.0/.
 */

@namespace url("http://www.mozilla.org/keymaster/gatekeeper/there.is.only.xul");

#ehh-helpbox, #ehh-commandlabel, #ehh-viewsource
{
  margin: 10px;
  padding: 5px;
}

#ehh-helpbox row
{
  -moz-box-align: center;
}

#ehh-helpbox .key:not(:empty)
{
  font-weight: bold;
  margin: 2px 10px 2px 0px;
  border: 1px solid black;
  text-align: start;
  width: 30px;
  height: 30px;
  padding: 2px;
  background-color: white;
  color: black;
  border-radius: 3px;
}

#ehh-helpbox-title
{
  font-size: 130%;
  margin-bottom: 10px;
}

#ehh-commandlabel
{
  font-size: 150%;
}

#ehh-commandlabel-key:not(:empty), #ehh-commandlabel-alternativeKey:not(:empty)
{
  font-weight: bold;
  margin: 0px 10px 0px 0px;
  border: 1px solid black;
  text-align: start;
  width: 45px;
  height: 45px;
  padding: 2px;
  background-color: white;
  color: black;
  border-radius: 4px;
}

#ehh-viewsource
{
  max-width: none;
}

#ehh-viewsource description,
#ehh-viewsource hbox,
#ehh-viewsource vbox
{
  margin: 0px;
}

#ehh-viewsource .elementBox
{
  border: 1px solid #CCCCCC;
  margin: 5px;
}

#ehh-viewsource .text,
#ehh-viewsource .cdata,
#ehh-viewsource .comment
{
  margin: 0px 5px;
}

#ehh-viewsource .tagName
{
  font-weight: bold;
  color: #FF0000;
}

#ehh-viewsource .attrName
{
  margin-left: 5px;
  color: #00FF00;
}

#ehh-viewsource .attrValue
{
  color: #0000FF;
}

#ehh-viewsource .comment
{
  color: #808080;
}
